//! Litera value parser.  
//! リテラル値パーサー。  

use crate::model::{
    layer110::{Token, TokenType},
    layer210::LiteralValue,
};
use crate::parser::phase200::{
    error,
    layer210::{LiteralValueP, PResult},
};
use casual_logger::Table as LogTable;

impl Default for LiteralValueP {
    fn default() -> Self {
        LiteralValueP {
            buffer: Some(LiteralValue::default()),
        }
    }
}
impl LiteralValueP {
    pub fn flush(&mut self) -> Option<LiteralValue> {
        if let Some(literal_value) = &self.buffer {
            let m = Some(literal_value.clone()); // TODO トリム要らないのでは。
            self.buffer = None;
            return m;
        }
        None
    }
    /// # Arguments
    ///
    /// * `tokens` - Tokens contains look ahead.  
    ///             先読みを含むトークン。  
    /// # Returns
    ///
    /// * `PResult` - Result.  
    ///                             結果。
    pub fn parse(&mut self, tokens: (Option<&Token>, Option<&Token>, Option<&Token>)) -> PResult {
        let token0 = tokens.0.unwrap();
        match token0.type_ {
            TokenType::AlphabetCharacter
            | TokenType::AlphabetString
            | TokenType::Colon
            | TokenType::Dot
            | TokenType::Hyphen
            | TokenType::NumeralString
            | TokenType::Plus
            | TokenType::Underscore => {
                let m = self.buffer.as_mut().unwrap();
                m.push_token(&token0);

                // Look-ahead.
                // 先読み。
                if let Some(token1) = tokens.1 {
                    match token1.type_ {
                        TokenType::AlphabetCharacter
                        | TokenType::AlphabetString
                        | TokenType::Colon
                        | TokenType::Dot
                        | TokenType::Hyphen
                        | TokenType::NumeralString
                        | TokenType::Plus
                        | TokenType::Underscore => PResult::Ongoing,
                        _ => PResult::End,
                    }
                } else {
                    PResult::End
                }
            }
            _ => return error(&mut self.log(), tokens, "literal_value_p.rs.38."),
        }
    }

    /// Log.  
    /// ログ。  
    pub fn log(&self) -> LogTable {
        let mut t = LogTable::default().clone();
        if let Some(m) = &self.buffer {
            t.str("buffer", &m.to_string());
        }
        t
    }
}
